declare type Fn<Args extends any[]> = (...args: Args) => any;
declare type AnyFn = Fn<any[]>;
declare type Listener<K extends string, Args extends any[]> = (name: K, listener: (...args: Args) => any, ...rest: any[]) => any;
declare type AnyListener = Listener<string, any[]>;
declare type Listenable = {
    addEventListener: AnyListener;
    removeEventListener?: AnyListener;
} | {
    addListener: AnyListener;
    removeListener?: AnyListener;
} | {
    on: AnyListener;
    off?: AnyListener;
} | {
    subscribe: AnyListener;
    unsubscribe?: AnyListener;
};
declare type EventName<O extends Listenable> = O extends Window ? keyof WindowEventMap : O extends Document ? keyof DocumentEventMap : O extends HTMLElement ? keyof HTMLElementEventMap : O extends WebSocket ? keyof WebSocketEventMap : O extends XMLHttpRequest ? keyof XMLHttpRequestEventMap : O extends {
    addEventListener: AnyFn;
} ? Parameters<O['addEventListener']>[0] : O extends {
    addListener: AnyFn;
} ? Parameters<O['addListener']>[0] : O extends {
    on: AnyFn;
} ? Parameters<O['on']>[0] : O extends {
    subscribe: AnyFn;
} ? Parameters<O['subscribe']>[0] : string;
declare type LinterBy<O extends Listenable, K extends string> = O extends Window ? K extends keyof WindowEventMap ? Fn<[WindowEventMap[K]]> : never : O extends Document ? K extends keyof DocumentEventMap ? Fn<[DocumentEventMap[K]]> : never : O extends HTMLElement ? K extends keyof HTMLElementEventMap ? Fn<[HTMLElementEventMap[K]]> : never : O extends WebSocket ? K extends keyof WebSocketEventMap ? Fn<[WebSocketEventMap[K]]> : never : O extends XMLHttpRequest ? K extends keyof XMLHttpRequestEventMap ? Fn<[XMLHttpRequestEventMap[K]]> : never : O extends {
    addEventListener: Listener<K, infer Args>;
} ? Fn<Args> : O extends {
    addListener: Listener<K, infer Args>;
} ? Fn<Args> : O extends {
    on: Listener<K, infer Args>;
} ? Fn<Args> : O extends {
    subscribe: Listener<K, infer Args>;
} ? Fn<Args> : AnyFn;
declare type Adder = <O extends Listenable, K extends EventName<O>>(o: O, name: K, listener: LinterBy<O, K>, ...params: any[]) => ClearAll;
declare type ClearAll = AnyFn & {
    add: Adder;
};
export declare function add(): ClearAll;
export declare function add<O extends Listenable, K extends EventName<O>>(o: O, name: K, listener: LinterBy<O, K>, ...params: any[]): ClearAll;
export default add;
